package com.engine.organization.mapper.job;

import com.engine.organization.entity.job.dto.JobListDTO;
import com.engine.organization.entity.job.po.JobPO;
import org.apache.ibatis.annotations.Param;

import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * @description: TODO
 * @author:dxfeng
 * @createTime: 2022/05/24
 * @version: 1.0
 */
public interface JobMapper {

    /**
     * 根据搜索条件查询数据
     *
     * @return
     */
    List<JobListDTO> listByFilter(JobPO jobPO);

    /**
     * 展示所有列表数据
     *
     * @return
     */
    List<JobListDTO> listNoFilter();

    /**
     * 查询所有数据
     *
     * @return
     */
    List<JobPO> listAll();

    /**
     * 根据ID查询元素
     *
     * @param id
     * @return
     */
    JobPO getJobById(@Param("id") Long id);

    /**
     * 根据父ID查询元素
     *
     * @param pid
     * @return
     */
    List<JobPO> getJobsByPid(@Param("pid") Long pid);

    /**
     * 浏览按钮展示数据用
     *
     * @param ids
     * @return
     */
    List<Map<String, Object>> listJobsByIds(@Param("ids") Collection<Long> ids);

    /**
     * 根据编号查询数据
     *
     * @param jobNo
     * @return
     */
    List<JobPO> listByNo(@Param("jobNo") String jobNo);

    /**
     * 根据所属部门查询数据
     *
     * @param departmentId
     * @return
     */
    List<JobPO> listJobsByDepartmentId(@Param("departmentId") long departmentId);

    /**
     * 添加数据
     *
     * @param jobPO
     * @return
     */
    int insertIgnoreNull(JobPO jobPO);


    /**
     * 更新主表内容
     *
     * @param jobPO
     * @return
     */
    int updateBaseJob(JobPO jobPO);

    /**
     * 更新禁用标识
     *
     * @param id
     * @param forbiddenTag
     * @return
     */
    int updateForbiddenTagById(@Param("id") long id, @Param("forbiddenTag") int forbiddenTag);

    /**
     * 批量删除
     *
     * @param ids
     */
    int deleteByIds(@Param("ids") Collection<Long> ids);

    /**
     * 查询所有被引用的ID
     *
     * @return
     */
    List<String> listUsedId();
}
